@@ -8,6 +8,7 @@ import logging
 import logging.handlers
 import argparse
 from datetime import datetime
+import dateutil.parser as dp
 import pytz
 import requests
 
@@ -57,8 +58,11 @@ def get_args():
   parser = argparse.ArgumentParser()
   parser.add_argument("-r", "--repository_url", help="URL of the github repository upon which this logger is being run", default='', required=False)
   parser.add_argument("-t", "--event_type", help="The type of event that is triggering this logger to be run, e.g. 'push' or 'pull_request', from github.event_type.", default='', required=False)
-  parser.add_argument("-i", "--inputfile", help="filename of JSON array of commits (typically saved from GitHub Action context variable, github.event.commits)", default='', required=True)
-  parser.add_argument("-o", "--outputfile", help="filename where to store the CSV output with git stats for each commit", default='', required=True)
+  parser.add_argument("-d", "--event_date", help="The date the event is being triggered.", default='', required=False)
+  parser.add_argument("-un", "--user_name", help="The name of the user performing the event.", default='', required=False)
+  parser.add_argument("-ue", "--user_email", help="The email of the user performing the event.", default='', required=False)
+  parser.add_argument("-i", "--inputfile", help="filename of JSON array of commits (typically saved from GitHub Action context variable, github.event.commits, during push events.)", default='', required=False)
+  parser.add_argument("-o", "--outputfile", help="filename where to store the CSV output with git stats for each commit.", default='', required=True)
   parser.add_argument("-u", "--url", help="The URL of the web app where the commit stats should be sent.", default='')
   parser.add_argument("-x", "--exclusions", help='A comma-separated string of files to exclude, e.g. --excusions "foo.zip, *.jpg, *.json" ', default=','.join(exclusions))
   parser.add_argument("-v", "--verbose", help="Whether to output debugging info", default=False, action="store_true")
@@ -82,13 +86,7 @@ def get_commit_ids(commit_datafile):
     # print(f'commit_ids: {commit_ids}')
     return commit_ids
 
-def get_commit_data(commit_id, exclusions):
-  '''
-  Get git stats for a commit.
-  @param commit_id: The commit id.
-  @param exclusions: The files to exclude from the git stats.
-  @return: A dictionary containing the git stats for the commit.
-  '''
+def get_git_output(commit_id, exclusions):
   # print(f'commit id: {commit_id}')
   date_format = r'--date=unix' #format:"%m/%d/%Y %H:%M" # formatted in a way that works well in Google Sheets
   cmd = f"git show {date_format} --shortstat {commit_id} {exclusions}"
@@ -98,9 +96,18 @@ def get_commit_data(commit_id, exclusions):
   git_log_out = git_log_out.decode('UTF-8') # convert bytes to string
   # print(f'git output: {git_log_out}')
   # print(f'git error: {git_log_err}')
+  return git_log_out.strip(), git_log_err.strip()
 
+def get_commit_data(git_log_out):
+  '''
+  Get git stats for a commit.
+  @param commit_id: The commit id.
+  @param exclusions: The files to exclude from the git stats.
+  @return: A dictionary containing the git stats for the commit.
+  '''
   # parse git commit log
   commit_data = {} # start off blank
+  git_log_out = re.sub("(\n {4}(.+)\n)+", r"\1", git_log_out) # remove multi-line commit messages
   m = re.match(r"commit ([a-zA-Z0-9]+).*\nAuthor:\s(.*)\s<((.*))>.*\nDate:\s(.*)\n\n(.*)\n\n(.*?(\d+) file[s]? changed)?(.*?(\d+) insertion[s]?)?(.*?(\d+) deletion[s]?)?", git_log_out)
   if not m is None:
     # basic commit info
@@ -110,10 +117,7 @@ def get_commit_data(commit_id, exclusions):
     commit_data['date'] = m.groups(0)[4].strip()
     commit_data['message'] = m.groups(0)[5].replace('[,"]', '').strip() # remove any quotes and commas to make a valid csv
     # fix the date
-    local_tz = pytz.timezone("America/New_York")
-    utc_time = datetime.utcfromtimestamp(int(commit_data['date'])) # convert unix time to utc datetime
-    tz_time = utc_time.replace(tzinfo=pytz.utc).astimezone(local_tz) # convert to NY time
-    commit_data['date'] = tz_time.strftime('%m/%d/%Y %H:%M') # formatted as nice string good for Google Sheets date field
+    commit_data['date'] = fix_date(commit_data['date'])
     # stats
     commit_data['files'] = m.groups(0)[7].strip()
     commit_data['additions'] = m.groups(0)[9].strip() if len(m.groups(0)) > 9 and type(m.groups(0)[9])==str else 0
@@ -121,6 +125,27 @@ def get_commit_data(commit_id, exclusions):
   # print(f'commit_data: {commit_data}')
   return commit_data
 
+def fix_date(bad_date):
+  '''
+  Fix the date format to be easier to parse by Google Sheets
+  @param date: The date to fix, either in unix time or ISO format
+  @return: The fixed date in a format that works well in Google Sheets
+  '''
+  try:
+    # date from git commits is in unix time
+    bad_date_int = int(bad_date)
+  except ValueError:
+    # date from github actions context is in ISO format
+    iso_date = bad_date
+    parsed_t = dp.parse(iso_date)
+    bad_date_int = int(parsed_t.strftime('%s'))
+
+  local_tz = pytz.timezone("America/New_York")
+  utc_time = datetime.utcfromtimestamp(bad_date_int) # convert unix time to utc datetime
+  tz_time = utc_time.replace(tzinfo=pytz.utc).astimezone(local_tz) # convert to NY time
+  good_date = tz_time.strftime('%m/%d/%Y %H:%M') # formatted as nice string good for Google Sheets date field
+  return good_date
+
 def verboseprint(verbose, *args):
   '''
   Print only if verbose flag is set.
@@ -139,8 +164,6 @@ def main():
   # set up logging
   logger = setup_logging(args.outputfile)
 
-  commit_ids = get_commit_ids(args.inputfile) # get the ids of all commits from the json file
-
   # set up exclusions
   exclusions = '-- . ' + ' '.join(["':(exclude,glob){}'".format(x) for x in args.exclusions]) # put the exclusions in the format git logs uses
   # print(f'exclusions: {exclusions}')
@@ -148,38 +171,80 @@ def main():
   # write the CSV heading line
   logger.info('repository_url,event_type,commit_id,commit_author_name,commit_author_email,commit_date,commit_message,commit_files,commit_additions,commit_deletions')
   
-  # iterate over commit ids and add each to a list
-  commits_list = [] # start it off blank
-  for commit_id in commit_ids:
-    
-    # get git stats for this commit
-    commit_data = get_commit_data(commit_id, exclusions) 
+  events_list = [] # start it off blank
 
-    # add repository url, if present
-    if args.repository_url:
-      verboseprint(args.verbose, f'repository_url: {args.repository_url}')
-      commit_data['repository_url'] = args.repository_url
+  # get pull request data, if any supplied
+  if args.event_type in ['pull_request_opened', 'pull_request_merged', 'pull_request_closed']:
+    # get pull request data
+    pull_request_data = {
+      'repository_url': args.repository_url,
+      'event_type': args.event_type,
+      'id': '',
+      'author_name': args.user_name,
+      'author_email': args.user_email,
+      'date': fix_date(args.event_date),
+      'message': '',
+      'files': '',
+      'additions': '',
+      'deletions': ''
+    }
 
-    # add event type, if present
-    if args.event_type:
-      verboseprint(args.verbose, f'event_type: {args.event_type}')
-      commit_data['event_type'] = args.event_type
+    # add this pull request to the list
+    events_list.append(pull_request_data)
 
-    # add this commit to the list
-    commits_list.append(commit_data) 
-    
-    # log it to the csv data file
-    logger.info(f'{args.repository_url},{commit_data["event_type"]},{commit_data["id"]},{commit_data["author_name"]},{commit_data["author_email"]},{commit_data["date"]},"{commit_data["message"]}",{commit_data["files"]},{commit_data["additions"]},{commit_data["deletions"]}')
+  # get push data, if any supplied
+  elif args.event_type in ['push']:
+
+    # get ids of each commit, if any supplied
+    commit_ids = []
+    if args.inputfile:
+      commit_ids = get_commit_ids(args.inputfile) # get the ids of all commits from the json file
+
+    # iterate over commit ids, if any, and add each to a list
+    for commit_id in commit_ids:
+
+      # get git output for this commit
+      git_log_out, git_log_err = get_git_output(commit_id, exclusions)
+      verboseprint(args.verbose, f'git output: {git_log_out}') # raw output
+      verboseprint(args.verbose, f'git err: {git_log_err}') # any errors
+
+      # handle missing commit data... this seems to occur with merge commits
+      if git_log_out == '':
+        verboseprint(args.verbose, f'empty git output... skipping')
+        continue
+
+      # parse git output
+      commit_data = get_commit_data(git_log_out) 
+      if not bool(commit_data):
+        verboseprint(args.verbose, f'empty parsed data... skipping')
+        continue
+      verboseprint(args.verbose, f'parsed git stats: {commit_data}')
+
+      # add repository url, if present
+      if args.repository_url:
+        # verboseprint(args.verbose, f'repository_url: {args.repository_url}')
+        commit_data['repository_url'] = args.repository_url
+
+      # add event type, if present
+      if args.event_type:
+        # verboseprint(args.verbose, f'event_type: {args.event_type}')
+        commit_data['event_type'] = args.event_type
+
+      # add this commit to the list
+      events_list.append(commit_data) 
+      
+      # log it to the csv data file
+      logger.info(f'{args.repository_url},{commit_data["event_type"]},{commit_id},{commit_data["author_name"]},{commit_data["author_email"]},{commit_data["date"]},"{commit_data["message"]}",{commit_data["files"]},{commit_data["additions"]},{commit_data["deletions"]}')
 
   # debugging print
-  verboseprint(args.verbose, f'commits_list: {commits_list}')
+  verboseprint(args.verbose, f'events list: {events_list}')
 
   # send the data to the web app URL, if any was supplied
   if args.url:
     # convert the list of commits to a JSON string
-    commits_json = json.dumps(commits_list)
+    commits_json = json.dumps(events_list)
     # send the data to the web app in a POST request
-    r = requests.post(args.url, json=commits_list)
+    r = requests.post(args.url, json=events_list)
     verboseprint(args.verbose, 'web app response: ', r.status_code, r.reason, r.content, r.text) # really the one
 
 if __name__ == "__main__":
